@@ -7,7 +7,7 @@ RDF::Query - A complete SPARQL 1.1 Query and Update implementation for use with
 
 =head1 VERSION
 
-This document describes RDF::Query version 2.912.
+This document describes RDF::Query version 2.913.
 
 =head1 SYNOPSIS
 
@@ -136,7 +136,7 @@ if (! Log::Log4perl::initialized()) {
 }
 
 no warnings 'numeric';
-use RDF::Trine 0.135;
+use RDF::Trine 1.004;
 require RDF::Query::Functions;	# (needs to happen at runtime because some of the functions rely on RDF::Query being fully loaded (to call add_hook(), for example))
 								# all the built-in functions including:
 								#     datatype casting, language ops, logical ops,
@@ -158,7 +158,7 @@ use RDF::Query::Plan;
 
 our ($VERSION, $DEFAULT_PARSER);
 BEGIN {
-	$VERSION		= '2.912';
+	$VERSION		= '2.913';
 	$DEFAULT_PARSER	= 'sparql11';
 }
 
@@ -238,7 +238,31 @@ sub new {
 	my $update	= ((delete $options{update}) ? 1 : 0);
 	my $pclass	= $names{ $lang } || $uris{ $languri } || $names{ $DEFAULT_PARSER };
 	my $parser	= $pclass->new( %pargs );
-	my $parsed	= $parser->parse( $query, $base_uri, $update );
+	my $parsed;
+	
+	if (ref($query) and $query->isa('RDF::Query::Algebra')) {
+		my $method	= 'SELECT';
+		$method		= 'ASK' if ($query->isa('RDF::Query::Algebra::Ask'));
+		$method		= 'CONSTRUCT' if ($query->isa('RDF::Query::Algebra::Construct'));
+		my @vars	= map { RDF::Query::Node::Variable->new($_) } _uniq($query->potentially_bound);
+		if ($method eq 'SELECT') {
+			unless ($query->isa('RDF::Query::Algebra::Project')) {
+				$query	= RDF::Query::Algebra::Project->new($query, \@vars);
+			}
+		}
+		$parsed	= {
+					method		=> $method,
+					triples		=> [$query],
+					sources		=> [],
+					base		=> $base_uri,
+					options		=> {},
+					star		=> 0,
+					variables	=> \@vars,
+				};
+		$query	= $query->as_sparql;
+	} else {
+		$parsed	= $parser->parse( $query, $base_uri, $update );
+	}
 	
 	my $self	= $class->_new(
 					base_uri		=> $base_uri,
@@ -454,7 +478,7 @@ sub execute {
 =item C<< execute_plan ( $plan, $context ) >>
 
 Executes the query plan generated by the C<<prepare>> method using the supplied
-L<RDF::Query::ExecutionContext>> object. Return value(s) are the same as for the
+L<RDF::Query::ExecutionContext> object. Return value(s) are the same as for the
 C<<execute>> method.
 
 =cut
